var assert = require("assert")
	, fs = require('fs')
	, util = require('util')
	, Marc = require('../index')
	, _ = require('underscore');

suite('Array', function() {

	setup(function(){});

	suite('Marc.Reader', function(){

		test('should return false when file is not exists', function(){
			var reader = Marc.Reader;
			assert.equal(false, reader.read('./test/sources/not.exist.marc'));
		});

		test('should return leader correctly', function(){
			var reader = Marc.Reader;
			var record = reader.read('./test/sources/1.marc');
			assert.equal("00480na  #2200181   4500", record.leader());
		});

		test('should extract record info correctly', function(){
			var reader = Marc.Reader;
			var record = reader.read('./test/sources/1.marc');
			var recordinfo = record.info();

			assert.equal("n", recordinfo.record_status());
			assert.equal("a", recordinfo.type_of_record());
			assert.equal(" ", recordinfo.bibliographic_level());
			assert.equal(" ", recordinfo.type_of_control());
			assert.equal("#", recordinfo.character_coding_scheme());
			assert.equal(2, recordinfo.indicator_count());
			assert.equal(2, recordinfo.subfield_code_count());
			
			assert.equal(" ", recordinfo.encoding_level());
			assert.equal(" ", recordinfo.descriptive_cataloging_form());
			assert.equal(" ", recordinfo.multipart_resource_record_level());
			assert.equal(4, recordinfo.length_of_length_field());
			assert.equal(5, recordinfo.length_of_starting_char_position());
			assert.equal(0, recordinfo.length_of_implementation_defined_portion());
		});

		test('should extract leader info correctly', function(){
			var reader = Marc.Reader;
			var record = reader.read('./test/sources/1.marc');
			
			assert.equal(480, record.recordlength());
			assert.equal(181, record.baseaddress());
		});

		test('should identify control fields correctly', function(){

			var reader = Marc.Reader;
			var record = reader.read('./test/sources/1.marc');

			assert.equal("3874", record.field(1).value());
			assert.equal("Milas", record.field(3).value());
			assert.equal("20120703101042.0", record.field(5).value());
		});

		test('should identify data fields correctly', function(){
			var reader = Marc.Reader;
			var record = reader.read('./test/sources/1.marc');
			
			assert.equal(" ", record.field(20).ind1());
			assert.equal(" ", record.field(20).ind2());
			assert.equal("978-975-6311-51-6", record.field(20).subfield('a').value());
			
			assert.equal(" ", record.field(41).ind1());
			assert.equal(" ", record.field(41).ind2());
			assert.equal("tur", record.field(41).subfield('a').value());

			assert.equal(" ", record.field(90).ind1());
			assert.equal(" ", record.field(90).ind2());
			assert.equal("346.561 DEM 2012", record.field(90).subfield('a').value());

			assert.equal(" ", record.field(99).ind1());
			assert.equal(" ", record.field(99).ind2());
			assert.equal("346.561 DEM 2012", record.field(99).subfield('a').value());

			assert.equal(" ", record.field(100).ind1());
			assert.equal(" ", record.field(100).ind2());
			assert.equal("Demirel, Salim; Murat Arapgirli", record.field(100).subfield('a').value());

			assert.equal(" ", record.field(245).ind1());
			assert.equal(" ", record.field(245).ind2());
			assert.equal("Aşırı Düşük Teklif Sorgulama Rehberi", record.field(245).subfield('a').value());

			assert.equal(" ", record.field(250).ind1());
			assert.equal(" ", record.field(250).ind2());
			assert.equal("1. bs.", record.field(250).subfield('a').value());

			assert.equal(" ", record.field(260).ind1());
			assert.equal(" ", record.field(260).ind2());
			assert.equal("Ankara", record.field(260).subfield('a').value());
			assert.equal("Türkiye Belediyeler Birliği", record.field(260).subfield('b').value());
			assert.equal("2012", record.field(260).subfield('c').value());

			assert.equal(" ", record.field(300).ind1());
			assert.equal(" ", record.field(300).ind2());
			assert.equal("186 s.: tbl.; 24 cm.", record.field(300).subfield('a').value());

			assert.equal(" ", record.field(650).ind1());
			assert.equal(" ", record.field(650).ind2());
			assert.equal("Kamu Hukuku.Kamu Yönetimi", record.field(650).subfield('a').value());
		});

		test('should return false when value() of data fields', function(){

			var reader = Marc.Reader;
			var record = reader.read('./test/sources/1.marc');

			assert.equal(false, record.field(20).value());
			assert.equal(false, record.field(41).value());
			assert.equal(false, record.field(90).value());
			assert.equal(false, record.field(99).value());
			assert.equal(false, record.field(100).value());
			assert.equal(false, record.field(245).value());
			assert.equal(false, record.field(250).value());
			assert.equal(false, record.field(260).value());
			assert.equal(false, record.field(300).value());
			assert.equal(false, record.field(650).value());
		});

		test('should return false if non-exist field is called', function(){

			var reader = Marc.Reader;
			var record = reader.read('./test/sources/1.marc');

			assert.equal(false, record.field(21));
			assert.equal(false, record.field(22));
			assert.equal(false, record.field(46));
			assert.equal(false, record.field(333));
		});

		test('should read one file containing multiple marcs', function(){
			/*
			var reader = new Marc.Reader({
				multiple: true
			});
			reader.read('./test/sources/multiple.marc');
			*/
		});

		test('should read json hash format', function(){
			
			var reader = Marc.Reader;
			var record = reader.read(JSON.parse(fs.readFileSync('./test/sources/loc_jsonhash.json').toString()));

			assert.equal(record.field(1).value(), "4191015");
			assert.equal(record.field(5).value(), "20120705141424.0");
			assert.equal(_.size(record.field(100, "a")), 1);
			assert.equal(_.first(record.field(100, "a")).value(), "Wackernagel, Martin,");
			assert.equal(record.leader(), "01266cam a2200325 i 4500");
		});
	});
});